var xhtmlns="http://www.w3.org/1999/xhtml";
function CompareGridItem(grid, div) {
    this.grid = grid;
    this.div = div;
    this.div.style.background = 'gray';
    this.div.style.cursor = 'pointer';
    this.mousedown = false;
    this.moved = false;

    // remove all current items
    while (this.div.firstChild) {
        this.div.removeChild(this.div.firstChild);
    }

    // set dragging handlers
    var t = this;
/*
    this.div.onselectstart = function(evt) { return false; }
*/
    this.div.onmouseout = function(evt) { return t.onmouseout(evt); }
    this.div.onmouseup = function(evt) { return t.onmouseup(evt); }
/*
    this.div.onmousemove = function(evt) { t.onmousemove(evt); }
*/
    this.div.onselectstart = function(evt) { evt.preventDefault(); return false; }
    this.div.onmousedown = function(evt) { return t.onmousedown(evt); }
    this.div.ondrag = function(evt) { return t.onmousemove(evt); }

    // create image tags
    this.spans = new Array();
    this.images = new Array();
    this.overlays = new Array();
    for (var n in this.grid.maxPageSizes) {
        var span = this.div.ownerDocument.createElementNS(xhtmlns, 'span');
        div.appendChild(span);
        span.style.display = 'inline-block';
        span.style.padding = '0px 0px 0px 1px';
        var img = CompareGridItem.createImage(this, n);
        this.spans[n] = span;
        this.images[n] = img;
        var overlay = CompareGridItem.createOverlay(this, n);
        span.appendChild(img);
        //span.appendChild(overlay);
        this.overlays[n] = overlay;
    }
    this.updateSizes();
}
CompareGridItem.prototype.getOverviewZoomLevel = function() {
    var g = this.grid;
    var cols = g.maxPageSizes.length;
    do {
         var rows = Math.ceil(g.maxPageSizes.length / cols);
         var entrywidth = this.div.clientWidth / cols;
         var p = g.maxPageSizes[0];
         var entryheight = p.width * entrywidth / p.height;
         var totalheight = entryheight * rows;
         if (totalheight > this.div.clientHeight) return 1/(cols + 1);
    } while (--cols > 1);
    return 0.5;
}
CompareGridItem.createImage = function(item, pagenumber) {
    var img = item.div.ownerDocument.createElementNS(xhtmlns, 'img');
    img.style.padding = '0px';
    img.style.border = '0px';
    img.style.margin = '0px';
    img.onclick = function(e) {
        item.mousedown = false;
        // clicking on an image toggles zoom level
        if (item.moved) return;
        item.moved = false;
        var g = item.grid;
        if (g.zoom == 1) {
            g.zoom = item.getOverviewZoomLevel();
        } else {
            g.zoom = 1;
        }
        g.changeZoom(0);
        item.div.scrollTop = item.spans[pagenumber].offsetTop;
        g.syncScrollBars(item.div);
    }
    return img;
}
CompareGridItem.createOverlay = function(item, pagenumber) {
    var img = item.div.ownerDocument.createElementNS(xhtmlns, 'div');
    img.style.padding = '0px';
    img.style.border = '0px';
    img.style.margin = '0px';
    img.style.position = 'float';
    img.style.clear = 'both'
//    img.style.top = '0px';
//    img.style.left = '0px';
    img.style.width = '50px';
    img.style.height = '50px';
    img.style.background = 'red';
    return img;
}
CompareGridItem.prototype.getZoom = function() {
    var zoom = this.grid.zoom;
    zoom = zoom * (this.div.clientWidth - 4);
    zoom = zoom / this.grid.maxPageSizes[0].width;
    if (zoom < 0.0001) zoom = 0.0001;
    return zoom;
}
CompareGridItem.prototype.showDocument = function(doc) {
    this.doc = doc;
    this.showImages(doc, this.images);
}
CompareGridItem.prototype.showOverlay = function(doc) {
    this.overlay = doc;
//    this.showImages(doc, this.overlays);
}
CompareGridItem.prototype.showImages = function(doc, images) {
    this.overlay = doc;
    var zoom = this.getZoom();
    for (var n in images) {
        var img = images[n];
        var p = doc.pages[n];
        if (p) {
            img.style.width = Math.floor(zoom * p.width) + 'px';
            img.style.height = Math.floor(zoom * p.height) + 'px';
            img.src = p.url;
        } else {
            img.style.width = 0;
            img.style.height = 0;
            img.src = '';
        }
    }
}
CompareGridItem.prototype.updateSizes = function() {
    var zoom = this.getZoom();
    for (var n in this.grid.maxPageSizes) {
        var span = this.spans[n];
        var img = this.images[n];
        var w = Math.floor(zoom * this.grid.maxPageSizes[n].width);
        var h = Math.floor(zoom * this.grid.maxPageSizes[n].height);
        span.style.width = w + 'px';
        span.style.height = h + 'px';

        var img = this.images[n];
        var p;
        if (this.doc) {
            p = this.doc.pages[n];
        }
        var iw = 0;
        var ih = 0;
        if (p) {
            iw = Math.floor(zoom * p.width);
            ih = Math.floor(zoom * p.height);
        }
        img.style.width = iw + 'px';
        img.style.height = ih + 'px';
        img.style.paddingRight = (w-iw) + 'px';
        img.style.paddingBottom = (h-ih) + 'px';
    }
}
CompareGridItem.prototype.onmousemove = function(evt) {
    if (this.mousedown) {
        this.div.scrollTop += this.dragStartY - evt.clientY;
        this.div.scrollLeft += this.dragStartX - evt.clientX;
        this.dragStartY = evt.clientY;
        this.dragStartX = evt.clientX;
        this.div.style.cursor = 'move';
        this.moved = true;
    }
            evt.preventDefault();
}
CompareGridItem.prototype.onmouseout = function(evt) {
    this.mousedown = false;
    this.div.style.cursor = 'pointer';
}
CompareGridItem.prototype.onmouseup = function(evt) {
    this.mousedown = false;
    this.div.style.cursor = 'pointer';
}
CompareGridItem.prototype.aboveScrollBar = function(evt) {
    var l = this.div.style.left.substring(0, this.div.style.left.length-2);
    var t = this.div.style.top.substring(0, this.div.style.top.length-2);
    if (evt.clientX - l < this.div.clientWidth
                && evt.clientY - t  < this.div.clientHeight) {
        return false;
    }
    return true;
}
CompareGridItem.prototype.onmousedown = function(evt) {
    if (!this.aboveScrollBar(evt)) {
        this.dragStartX = evt.clientX;
        this.dragStartY = evt.clientY;
        this.mousedown = true;
        this.moved = false;
        if (typeof evt.preventDefault != 'undefined') {
       //     evt.preventDefault();
        }
    }
}
function CompareGrid(configuration, div) {
    this.config = configuration;
    this.div = div;
    this.zoom = 1;

    this.maxPageSizes = CompareGrid.getMaxPageSizes(this.config.documents);

    // create a grid
    var nrows = this.config.gridheight;
    var ncols = this.config.gridwidth;
    var doc = div.ownerDocument;
    this.grid = new Array();
    for (var n = 0; n < nrows; ++n) {
        this.grid[n] = new Array();
        for (var m = 0; m < ncols; ++m) {
            var d = doc.createElementNS(xhtmlns, 'div');
            d.style.position = 'absolute';
            d.style.overflow = 'scroll';
            var t = this;
            d.onscroll = function(e) {
                t.syncScrollBars(e.target);
            }
            this.div.appendChild(d);
            this.grid[n][m] = new CompareGridItem(this, d);
        }
    }

    // set initial pages
    for (var n in this.config.documents) {
        if (n / ncols < nrows) {
            var row = Math.floor(n/ncols);
            this.grid[row][n%ncols].showDocument(this.config.documents[n]);
        }
    }
    this.grid[0][0].showOverlay(this.config.documents[1]);
}
CompareGrid.getMaxPageSizes = function(documents) {
    var pages = new Array();
    var npages = 0;
    for (var i in documents) {
        if (npages < documents[i].pages.length) {
            npages = documents[i].pages.length;
        }
    }
    for (var i = 0; i < npages; ++i) {
        pages[i] = new Object();
        pages[i].width = 0;
        pages[i].height = 0;
        for (var j in documents) {
            if (documents[j].pages[i]) {
                if (documents[j].pages[i].width > pages[i].width) {
                    pages[i].width = documents[j].pages[i].width;
                }
                if (documents[j].pages[i].height > pages[i].height) {
                    pages[i].height = documents[j].pages[i].height;
                }
            }
        }
    }
    return pages;
}
CompareGrid.prototype.syncScrollBars = function(div) {
    var top = div.scrollTop;
    var left = div.scrollLeft;
    var nrows = this.config.gridheight;
    var ncols = this.config.gridwidth;
    for (var n = 0; n < nrows; ++n) {
        for (var m = 0; m < ncols; ++m) {
            var d = this.grid[n][m].div;
            if (d != div) {
                d.scrollTop = top;
                d.scrollLeft = left;
            }
        }
    }
}
CompareGrid.prototype.resize = function(width, height) {
    var nrows = this.config.gridheight;
    var ncols = this.config.gridwidth;
    for (var n = 0; n < nrows; ++n) {
        for (var m = 0; m < ncols; ++m) {
            var d = this.grid[n][m].div;
            var h = Math.floor(height / nrows);
            var w = Math.floor(width / ncols);
            d.style.top = n * h + 'px';
            d.style.height = h + 'px';
            d.style.left = m * w + 'px';
            d.style.width = w + 'px';
        }
    }
    this.changeZoom(0);
}
CompareGrid.prototype.changeZoom = function(zoomchange) {
    this.zoom = this.zoom * Math.pow(1.1, zoomchange);
    var nrows = this.config.gridheight;
    var ncols = this.config.gridwidth;
    for (var n = 0; n < nrows; ++n) {
        for (var m = 0; m < ncols; ++m) {
            this.grid[n][m].updateSizes();
        }
    }
}
CompareGrid.prototype.aboveScrollBar = function(evt) {
    return true;
}
