#!/usr/bin/python -Qwarnall

import sys, os, getopt, threading, subprocess, struct, glob, shutil

def die(message):
	print >> sys.stderr, str(message)
	exit(1)

def renderToPng(pdffile, outdir):
	out = os.path.join(outdir, os.path.basename(pdffile))
        print "pdftoppm -r 150 -png " + pdffile + " " + out
        os.system("pdftoppm -r 150 -png " + pdffile + " " + out)
	# get the dimensions of the output files
	pnginfo = []
	pngstruct = struct.Struct(">IIIIII")
	pnglist = glob.glob(out + '-*')
	if pnglist:
		pnglist.sort()
	for f in pnglist:
		fd = open(f, 'rb')
		data = fd.read(24)
		fd.close()
		(width, height) = pngstruct.unpack(data)[4:6]
		pnginfo.append([os.path.basename(f), width, height ])
	return pnginfo

def createCompareConfiguration(infolist):
	c = 'var configuration = {\n' \
		+ '  gridwidth: ' + str(len(infolist)) + ',\n' \
		+ '  gridheight: 1,\n' \
		+ '  documents: [\n'
	first = True
	for info in infolist:
		if not first:
			c += ',\n'
		first = False
		c += "    {\n      name:'dummy',\n      pages: [\n"
		firstPage = True
		for page in info:
			if not firstPage:
				c += ',\n'
			firstPage = False
			c += "        { url: '" + page[0] + "', width: " \
				+ str(page[1]) + ", height: " + str(page[2]) \
				+ " }"
		c += "\n      ]\n    }"
	c += '\n  ]\n};\n'
	return c

def kill_proc(proc):
	try:
		proc.kill()
	except:
		pass

def renderToPdf(docxfile, pdffile, script):
	env = os.environ
	env["DISPLAY"] = ":2"
	env["KDE_DEBUG"] = "0"

	proc = subprocess.Popen([script, docxfile, pdffile], env=env)
	t = threading.Timer(6, kill_proc, {proc: proc})
	t.start()
	proc.wait()

def createComparison(docxfile, outdir):
	docxpdf1 = os.path.join(outdir, os.path.basename(docxfile) + '1.pdf')
	docxpdf2 = os.path.join(outdir, os.path.basename(docxfile) + '2.pdf')
	renderToPdf(docxfile, docxpdf1, "./toPdf-LibreOffice.py")
	renderToPdf(docxfile, docxpdf2, "./toPdf-Calligra.py")
	docxpdfpnglist1 = renderToPng(docxpdf1, outdir)
	docxpdfpnglist2 = renderToPng(docxpdf2, outdir)
	configstring = createCompareConfiguration([docxpdfpnglist1, docxpdfpnglist2])
	f = open(os.path.join(outdir, 'configuration.js'), 'w')
	f.write(configstring)
	f.close()
	shutil.copy(os.path.join(sys.path[0], 'slidecompare.html'),
		os.path.join(outdir, 'index.html'))
	shutil.copy(os.path.join(sys.path[0], 'comparegrid.js'), outdir)

def scanDirectory(rootdir, extensions):
	if os.path.isfile(rootdir):
		return [rootdir]

	filext = map(lambda e: "." + e, extensions)
	list = []
	for root, directories, files in os.walk(rootdir):
		for file in files:
			if file.startswith('.'):
				continue
			if any(map(lambda e: file.endswith(e), filext)):
				list.append(os.path.join(root, file))
	return frozenset(list)

if __name__ == '__main__':
	if len(sys.argv) != 3:
		die('Usage: ' + sys.argv[0] + ' indir outdir')
	indir = sys.argv[1]
	outdir = sys.argv[2]

	if not os.path.isdir(indir):
		die(indir + ' is not a directory')
	try:
		os.makedirs(outdir)
	except:
		pass
	if not os.path.isdir(outdir):
		die(outdir + ' is not a directory')
	if not os.access(outdir, os.W_OK):
		die(outdir + ' is not writable')

	docxfiles = scanDirectory(indir, ['docx', 'doc', 'odt'])

	displayproc = subprocess.Popen(["/usr/bin/Xephyr", "-noreset", ":2"])
	filelist = []
	for docx in docxfiles:
		reldir = os.path.relpath(os.path.splitext(docx)[0], indir)
		tgtdir = os.path.join(outdir, reldir)
		if not os.path.isdir(tgtdir):
			os.makedirs(tgtdir)
		createComparison(docx, tgtdir)
		filelist.append(reldir + '/' + os.path.basename(docx))
	displayproc.kill()

	# create overview page
	html = '<html><head><title>Easy compare overview</title></head><body>' \
		+ '<p>'
	filelist.sort()
	for f in filelist:
		reldir = os.path.dirname(f)
		html += '<a href="'+reldir+'/index.html">' + f + '</a><br/>\n'
	html += '</p></body></html>'
	htmlfile = os.path.join(outdir, 'index.html')
	f = open(htmlfile, 'w')
	f.write(html)
	f.close()

