#!/usr/bin/python -Qwarnall

# OpenOffice connector code
# Copyright (C) 2008 Mirko Nasato <mirko@artofsolving.com>
# Licensed under the GNU LGPL v2.1 - http://www.gnu.org/licenses/lgpl-2.1.html
# - or any later version.

import sys, os, getopt, time, traceback, subprocess, signal, zipfile, struct, glob, shutil

DEFAULT_OPENOFFICE_PORT = 8100
# this was needed to find uno
sys.path.append("/usr/lib/ooo-2.0/program") 

import uno
from os.path import abspath, isfile, splitext
from com.sun.star.beans import PropertyValue
from com.sun.star.task import ErrorCodeIOException
from com.sun.star.connection import NoConnectException

class UnoServer:
	""" Class that provides functionality to deal with the OpenOffice.org server instance. """

	def __init__(self, port=DEFAULT_OPENOFFICE_PORT):
		try:
			self.process = subprocess.Popen([
				'soffice',
				'-nologo', #don't show startup screen.
				'-minimized', #keep startup bitmap minimized.
				'-norestore', #suppress restart/restore after fatal errors.
				'-invisible', #no startup screen, no default document and no UI.
				'-headless',
				"-accept=socket,host=localhost,port=%s;urp;" % port
			], )
		except IOError:
			traceback.print_exc()
			raise

	def __del__(self):
		if hasattr(self,'process'):
			os.kill(self.process.pid, signal.SIGKILL)

class DocumentConverter:
	
	def __init__(self, port=DEFAULT_OPENOFFICE_PORT):
		self.unoserver = UnoServer()
		localContext = uno.getComponentContext()
		resolver = localContext.ServiceManager.createInstanceWithContext("com.sun.star.bridge.UnoUrlResolver", localContext)
		# try 10 seconds to connect
		tryStartTime = time.time()
		context = None
		while tryStartTime + 10 > time.time() and context == None:
			try:
				context = resolver.resolve("uno:socket,host=localhost,port=%s;urp;StarOffice.ComponentContext" % port)
			except NoConnectException:
				context = None
		if context == None:
			raise die("failed to connect to OpenOffice.org on port %s" % port)
		self.desktop = context.ServiceManager.createInstanceWithContext("com.sun.star.frame.Desktop", context)

	def convert(self, inputFile, outputFile):
		inputUrl = self._toFileUrl(inputFile)
		outputUrl = self._toFileUrl(outputFile)

		print inputUrl		
		document = self.desktop.loadComponentFromURL(inputUrl, "_blank", 0, self._toProperties(Hidden=True))
		try:
			document.refresh()
		except AttributeError:
			pass
		
		try:
			document.storeToURL(outputUrl, self._toProperties(FilterName='writer_pdf_Export'))
		finally:
			document.close(True)

	def _toFileUrl(self, path):
		return uno.systemPathToFileUrl(abspath(path))

	def _toProperties(self, **args):
		props = []
		for key in args:
			prop = PropertyValue()
			prop.Name = key
			prop.Value = args[key]
			props.append(prop)
		return tuple(props)

def die(message):
	print >> sys.stderr, str(message)
	exit(1)


if __name__ == '__main__':
	if len(sys.argv) != 3:
		die('Usage: ' + sys.argv[0] + ' infile outfile')
	infile = sys.argv[1]
	outfile = sys.argv[2]

	if not os.path.isfile(infile):
		die(infile + ' is not a file')
	outdir = os.path.dirname(outfile)
	try:
		os.makedirs(outdir)
	except:
		pass
	if not os.path.isdir(outdir):
		die(outdir + ' is not a directory')
	if not os.access(outdir, os.W_OK):
		die(outdir + ' is not writable')

	converter = DocumentConverter()    
	converter.convert(infile, outfile)


